<?php
/**
 * Copyright Intermesh
 *
 * This file is part of Group-Office. You should have received a copy of the
 * Group-Office license along with Group-Office. See the file /LICENSE.TXT
 *
 * If you have questions write an e-mail to info@intermesh.nl
 * @version $Id: imap.class.inc 4586 2010-04-13 15:06:13Z mschering $
 * @copyright Copyright Intermesh
 * @author Merijn Schering <mschering@intermesh.nl>
 * @package go.mail
 * @since Group-Office 1.0
 */

/**
 * Include RCF8222 class
 */
require_once($GLOBALS['GO_CONFIG']->class_path.'mail/RFC822.class.inc');
require_once($GLOBALS['GO_CONFIG']->class_path.'cache.class.inc.php');



/**
 * This class can be used to connect to IMAP or POP-3 servers
 * @version $Id: imap.class.inc 4586 2010-04-13 15:06:13Z mschering $
 * @author Merijn Schering <mschering@intermesh.nl>
 * @package go.mail
 * @access public
 */
class imap{
	/**
	 * The recource ID that points to the IMAP stream
	 *
	 * @var     		Recource ID
	 * @access  private
	 */
	var $conn;

	/**
	 * The number of messages in the current mailbox
	 *
	 * @var     		Int
	 * @access  private
	 */
	var $count;

	/**
	 * The number of unseen messages in the current mailbox
	 *
	 * @var     		Int
	 * @access  private
	 */
	var $unseen;


	/**
	 * Current message that is get with get_message();
	 *
	 * @var     		Array
	 * @access  private
	 */
	var $message;

	/**
	 * The type of the mailserver. IMAP or POP-3
	 *
	 * @var     		String
	 * @access  private
	 */
	var $servertype;

	/**
	 * The string to connect to the mailserver with imap_open();
	 *
	 * @var     		String
	 * @access  private
	 */
	var $connectstring;

	/**
	 * Array of mailboxes fetched with get_mailboxes();
	 *
	 * @var     		Array
	 * @access  private
	 */
	var $mailboxes;

	/**
	 * Currently opened mailbox
	 *
	 * @var     		String
	 * @access  private
	 */
	var $mailbox;

	/**
	 * Sorted message uid's returned by imap_sort();
	 *
	 * @var     		Array
	 * @access  private
	 */
	var $sort;

	/**
	 * Array with mailbox info fetched with imap_mailboxinfo()
	 *
	 * @var     		Array
	 * @access  private
	 */
	var $mailbox_info;

	/**
	 * Class to handle formatting of e-mail address strings.
	 *
	 * @var     		Object
	 * @access  private
	 */
	var $RFC822;
	
	
	var $touched_folders =array();
	

	/**
	 * Constructor. Initialises RFC822 class
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {

		global $GO_SECURITY;

		$this->RFC822 = new RFC822();
	}

	function __destruct(){		
			$this->close();
	}

	/**
	 * Opens a connection to server
	 *
	 * @param	string	$host					The hostname of the mailserver
	 * @param	string	$type					The type of the mailserver (IMAP or POP-3)
	 * @param	int 			$port 					The port to connect to
	 * @param	string	$username	The username
	 * @param	string	$password		The password
	 * @param	string	$mailbox			The mailbox to open
	 * @param	string	$flags					Connection flags (See PHP docs imap_open()
	 * @param	bool		$ssl						Connect in SSL mode or not
	 * @param	bool		$novalidate_cert						Don't validate SSL certificate
	 * @access public
	 * @return mixed	The recource ID on success or false on failure
	 */
	function open($host, $type, $port, $username, $password, $mailbox = "INBOX", $flags = null, $ssl = false, $novalidate_cert = false, $tls=false) {
		global $GO_CONFIG;

		$this->host=$host;
		$this->username=$username;

		$this->cache_key = $this->host.'.'.$this->username;

		$this->servertype = strtolower($type);
		$this->mailbox = $mailbox;
		
		$this->touched_folders[]=$this->mailbox;
		
		$this->connectstring = $host.":".$port."/".$this->servertype;

		if ($ssl) {
			$this->connectstring .= '/ssl';
		}

		if($tls)
		{
			$this->connectstring .= '/tls';
		}else
		{
			$this->connectstring .= '/notls';
		}

		if ($novalidate_cert) {
			$this->connectstring .= '/novalidate-cert';
		}
		
		//go_debug($this->connectstring);

		if ($flags != 0) {
			$this->conn = @imap_open("{".$this->connectstring."}".$this->mailbox, $username, $password, $flags, 1);
		} else {
			$this->conn = @imap_open("{".$this->connectstring."}".$this->mailbox, $username, $password);
		}
		
		//go_debug('Connected to IMAP server');
		return $this->conn;
	}
	
	/**
	 * Returns an array of integer values pertaining to the specified user mailbox. All values contain a key based upon the resource name, and a corresponding array with the usage and limit values within.
	 * 
	 * This function will return FALSE in the case of call failure, and an array of information about the connection upon an un-parsable response from the server. 
	 *
	 * @return unknown
	 */
	
	function get_quota(){		
		if(!$this->is_imap())
		{
			return false;
		}else
		{
			return @imap_get_quotaroot($this->conn, $this->mailbox);
		}		 
	}

	/**
	 * Return last error
	 *
	 * @access public
	 * @return String Last IMAP error
	 */

	function last_error()
	{
		$error = imap_last_error();
		$this->clear_errors();
		return $error;
	}

	/**
	 * Close the connection to the mailserver
	 *
	 * @access public
	 * @return void
	 */
	function close() {
		if(isset($this->conn) && $this->conn)
		{
			unset ($this->count);
			unset ($this->unseen);

			//for supressing notices that are not really errors
			$this->clear_errors();

			imap_close($this->conn);
			unset ($this->conn);
		}
	}

	function clear_errors()
	{
		imap_errors();
	}

	/**
	 * Count total and new messages on server
	 *
	 * @param	string	$mailbox	The mailbox to get status of
	 * @access public
	 * @return void
	 */
	function status($mailbox = false, $options=SA_UNSEEN) {
		if (!$mailbox)
		$mailbox = $this->mailbox;

		$status = imap_status($this->conn, "{".$this->connectstring."}".$mailbox, $options);
		return $status;
	}

	/**
	 * Count total and new messages on server
	 *
	 * @param	string	$mailbox	The mailbox to get status of
	 * @access public
	 * @return void
	 */
	function get_unseen($mailbox = false, $recursive=false) {
		if (!$mailbox)
		$mailbox = $this->mailbox;


		if(!$recursive)
		{
			$status = imap_status($this->conn, "{".$this->connectstring."}".$mailbox, SA_UNSEEN);
			return $status->unseen;
		}else
		{
			$unseen =0;
			//Sometimes INBOX is subscribed, sometimes not. So check it manually and skip it
			//if it's subscribed.
			if($mailbox == 'INBOX')
			{
				$status = imap_status($this->conn, "{".$this->connectstring."}INBOX", SA_UNSEEN);
				$unseen+=$status->unseen;
			}
			$folders = $this->get_subscribed($mailbox);
			foreach($folders as $folder)
			{
				if($folder['name'] != 'INBOX')
				{
					$status = imap_status($this->conn, "{".$this->connectstring."}".$folder['name'], SA_UNSEEN);
					$unseen+=$status->unseen;
				}
			}
			return $unseen;
		}
	}

	/**
	 * Check if this is an IMAP server
	 *
	 * @param	string	$mailbox	The mailbox to get status of
	 * @access public
	 * @return bool	 True if this is an IMAP server
	 */

	function is_imap() {
		if ($this->servertype == "imap") {
			return true;
		} else {
			return false;
		}
	}


	/**
	 * Sort message UID's into $this->sort (see imap_sort() PHP docs)
	 *
	 * @param	int	$sort_type	The column
	 * @param	string $reverse Reverse sorting (0 or 1)
	 * @param	string $search Search query
	 * @access public
	 * @return int	 Number of sorted messages
	 */
	function sort($sort_type = SORTDATE, $reverse = "1", $query = '') {

		$this->sort_type=$sort_type;
		$this->sort_reverse=$reverse;

		if ($query != '') {
			$this->sort = imap_sort($this->conn, $sort_type, $reverse, SE_UID+SE_NOPREFETCH, trim($query));
		} else {
			$this->sort = imap_sort($this->conn, $sort_type, $reverse, SE_UID+SE_NOPREFETCH);			
		}
	}

	function build_search_query($subject = '', $from = '', $to = '', $cc = '', $body = '', $before = '', $since = '', $before = '', $since = '', $flagged = '', $answered = '', $seen='') {
		$query = '';
		if ($subject != '') {
			$query .= 'SUBJECT "'.$subject.'" ';
		}
		if ($from != '') {
			$query .= 'FROM "'.$from.'" ';
		}
		if ($to != '') {
			$query .= 'TO "'.$to.'" ';
		}
		if ($cc != '') {
			$query .= 'CC "'.$cc.'" ';
		}
		if ($body != '') {
			$query .= 'BODY "'.$body.'" ';
		}

		if ($before != '') {
			$unix_before = Date::to_unixtime($before);
			$query .= 'BEFORE "'.date('d-M-Y', $unix_before).'" ';
		}

		if ($since != '') {
			$unix_since = Date::to_unixtime($since);
			$query .= 'SINCE "'.date('d-M-Y', $unix_since).'" ';
		}

		if ($flagged != '') {
			$query .= $flagged.' ';
		}

		if ($answered != '') {
			$query .= $answered.' ';
		}

		if ($seen != '') {
			$query .= $seen.' ';
		}

		return $query;
	}

	function get_message_uids($first, $offset, $sort_type = SORTDATE, $reverse = "1", $query = '')
	{
		//get the unseen and total messages		
		$status = $this->status($this->mailbox, SA_UNSEEN+SA_MESSAGES);
		if($status)
		{
			$this->unseen = $status->unseen;
			$this->count = $status->messages;
		}else
		{
			$this->unseen = $this->count = 0;
		}
		$this->query = $query;
		$this->first = $first;
		$this->offset = $offset;
		
		//sort the uid's
		$this->sort($sort_type, $reverse, $query);
		
		return $this->get_uids_subset($first, $offset);
	}
	
	function get_uids_subset($first, $offset)
	{
		$first=intval($first);
		$offset=intval($offset);
		
		$this->first = $first;
		$this->offset = $offset;
		
		$count = count($this->sort);

		$last = $first + $offset;
		if ($offset == 0 || ($last > $count)) {
			$last = $count;
		}

		$index = 0;
		$sorted_uids = array ();
		for ($i = $first; $i < $last; $i ++) {
			$sorted_uids[] = $this->sort[$i];
		}	
		return $sorted_uids;
	}
		
	function get_message_headers($uids)
	{		
		$messages = array();
		$get = implode(',', $uids);
			
		$overviews = imap_fetch_overview($this->conn, $get, FT_UID);
		foreach($overviews as $overview)
		{
			$message = $this->get_message($overview, false, true);			
			
			//$cached_message['folder_id']=$folder_id;
			$cached_message['uid']=$message['uid'];
			$cached_message['new']=$message['new'] ? '1' : '0';
			$cached_message['subject']=$message['subject'];
			$cached_message['size']=$message['size'];
			$cached_message['udate']=$message['udate'];
			$cached_message['attachments']=$message['attachments'] ? '1' : '0';
			$cached_message['flagged']=$message['flagged'] ? '1' : '0';
			$cached_message['answered']=$message['answered'] ? '1' : '0';
			$cached_message['priority']=$message['priority'];
			$cached_message['from']=$message['from'];
			$cached_message['reply_to']=$message['reply-to'];
			$cached_message['to']=implode(',', $message['to']);

			//this field will not be stored in cached by cached_imap.class.inc.php but
			//is used for filtering
			$cached_message['cc']=implode(',', $message['cc']);

			$cached_message['notification']=$message['notification'];
			$cached_message['content_type']=$message['content_type'];
			$cached_message['content_transfer_encoding']=$message['content_transfer_encoding'];

			$messages[$message['uid']]=$cached_message;
		}
		return $messages;
	}


	/**
	 * Get's all mime header values in an associative array
	 *
	 * @param String $header MIME headers
	 * @return Array with header values
	 */


	function get_header_values($header)
	{
		$header = str_replace("\r\n", "\n", $header);
		//commented with \h is better but only supported from 5.2.4
		//$header = preg_replace("/\n\h/", " ", $header);
		$header = preg_replace("/\n\s/", " ", $header);

		$lines = explode("\n", $header);

		$values = array();
		foreach($lines as $line)
		{
			$line = stripslashes($line);
			$colpos = strpos($line, ':');

			if($colpos)
			{
				$key = strtolower(trim(substr($line,0,$colpos)));
				$value = trim(substr($line, $colpos+1));

				$values[$key]=$value;
			}
		}

		$message["priority"] = isset($values['x-priority']) ? intval($values['x-priority']) : 3;
		$message['attachments']=false;

		if (!empty($values['content-type']) && preg_match("'([^/]*)/([^ ;\n\t]*)'i", $values['content-type'], $regs)) {
			$mtype = strtolower($regs[1]);
			$stype = strtolower($regs[2]);
		}else
		{
			$mtype = '';
			$stype = '';
		}
		if ($mtype != 'text' && $stype != 'alternative' && $stype != 'related' && !empty($mtype) && !empty($stype))
		{
			$message["attachments"] = true;
		}


		$RFC822 = new RFC822();

		$message["to"]=array();
		if(isset($values['to']))
		{
			$to = $RFC822->parse_address_list($values['to']);
			foreach($to as $address)
			{
				$message['to'][]=$RFC822->write_address($this->enc_utf8($address['personal']), $address['email']);
			}
		}

		$message["cc"]=array();
		if(isset($values['cc']))
		{
			$cc = $RFC822->parse_address_list($values['cc']);
			foreach($cc as $address)
			{
				$message['cc'][]=$RFC822->write_address($this->enc_utf8($address['personal']), $address['email']);
			}
		}
		
		$message["bcc"]=array();
		if(isset($values['bcc']))
		{
			$cc = $RFC822->parse_address_list($values['bcc']);
			foreach($cc as $address)
			{
				$message['bcc'][]=$RFC822->write_address($this->enc_utf8($address['personal']), $address['email']);
			}
		}

		$message['from']='';
		if(isset($values['from'])){
			$address = $RFC822->parse_address_list($values['from']);
			if(isset($address[0]))
				$message['from']=$RFC822->write_address($this->enc_utf8($address[0]['personal']), $address[0]['email']);
		}

		$message['reply-to']=$message['from'];
		if(isset($values['reply-to'])){
			$addresses = $RFC822->parse_address_list($values['reply-to']);
			if(isset($addresses[0])){
				$reply_to_addresses=array();
				while($address = array_shift($addresses)){
					$reply_to_addresses[]=$RFC822->write_address($this->enc_utf8($address['personal']), $address['email']);
				}
				
				$message['reply-to']=implode(',', $reply_to_addresses);
			}
		}


		if(isset($values['date'])){
		
			//a popular mail client sends the time incorrect. Here's a hack to make it work
			if(substr($values['date'],-2)=='UT')
			{
				$values['date'].='C';
			}			
			//some clients put a (+0100) behind the date.
			$values['date']=trim(preg_replace('/\(.+\)/', '', $values['date']));					
			
			//cut away the 2-character abbreviation and the comma
			if(preg_match('/^[a-zA-Z]{2} /', $values['date']))			
			{
				$values['date'] = substr($values['date'], 3);
				$values['date'] = str_replace(',','',$values['date']);
			}			

			$message['udate']=strtotime($values['date']);
		}else
		{
			//No date header?!? just use current time.

			//current time caused sync to send message over and over again.
			$message['udate']=0;//time();
		}	
		
		$message['subject'] = isset($values['subject']) ? $this->enc_utf8($values['subject']) : '';

		$message["notification"]=isset($values['disposition-notification-to']) ? $this->enc_utf8($values['disposition-notification-to']) : false;
		$message['content_type']=isset($values['content-type']) ? $values['content-type'] : false;
		$message['content_transfer_encoding']=isset($values['content-transfer-encoding']) ? $values['content-transfer-encoding'] : '';

		return $message;
	}


	function headerinfo($uid){
		return imap_headerinfo($this->conn, imap_msgno($this->conn,$uid));
	}


	function get_source($uid)
	{
		$header = imap_fetchheader($this->conn,$uid, FT_UID);
		if(!$header)
				return false;
	
		$body = imap_body($this->conn, $uid, FT_UID);
		if(!$body)
				return false;
		return $header.$body;
	}

	/**
	 * Get one message with the structure
	 *
	 * @param int $uid The unique identifier of the
	 * @param string $part Get a specific part of a message
	 * @access public
	 * @return array The E-mail message elements
	 */
	function get_message($uid, $fetchstructure=true, $nocache=false) {
		if ($this->conn) {
			unset ($this->message);

			if(is_object($uid))
			{
				$this->message['uid'] = $uid->uid;
				//$this->message['number'] = $uid->msgno;
				$this->message['new'] = (!$uid->seen && $this->is_imap());
				$this->message["flagged"] = $uid->flagged;
				$this->message["size"] = $uid->size;
				$this->message["answered"] = $uid->answered;
			}else
			{
				$this->message['uid'] = $uid;
				//$this->message['number'] = imap_msgno($this->conn, $uid);

				/*if(!@$headerinfo = imap_headerinfo($this->conn, $this->message['number']))
				{
					//message doesn't exist
					return false;
				}
				$this->message['new'] = (($headerinfo->Recent == 'N' || $headerinfo->Unseen == 'U')  && $this->is_imap());
				$this->message["flagged"] = $headerinfo->Flagged=='F';
				$this->message["size"] = $headerinfo->Size;
				$this->message["answered"] = $headerinfo->Answered=='A';*/
			}				
			$this->message["parts"] = array();

			if($fetchstructure)
			{
				$structure = imap_fetchstructure($this->conn, $this->message['uid'], FT_UID);
				if(!$structure){
					global $GO_LANGUAGE;
					require($GO_LANGUAGE->get_language_file('email'));
					throw new Exception($lang['email']['errorGettingMessage']."\n\n".$this->last_error());
				}

				//go_debug($structure);
				$this->get_parts($structure);
				$this->message['parts']=array_reverse($this->message['parts']);
			}

			$header = imap_fetchheader($this->conn, $this->message['uid'], FT_UID);
			if(!$header){
				require($GLOBALS['GO_LANGUAGE']->get_language_file('email'));
				throw new Exception($lang['email']['errorGettingMessage']."\n\n".$this->last_error());
			}

			$values = $this->get_header_values($header);
			
			$this->message = array_merge($this->message, $values);

			return $this->message;
		} else {
			return false;
		}
	}

	/**
	 * Get structured parts of a message
	 *
	 * @param array $mimeobj An array returned by imap_fetch_structure()
	 * @param string $section The current section of the message
	 * @access private
	 * @return void
	 */
	function get_parts($mimeobj, $section = 0, $parttype='') {

		if (isset ($mimeobj->type)) {
			$type = $this->get_mime_type($mimeobj->type);
		} else {
			$type = 'text';
		}

		$full_mime_type = $type."/".$mimeobj->subtype;
		$encoding = $this->get_encoding($mimeobj->encoding);

		//go_debug($full_mime_type.' '.$section);

		$name = '';

		if (isset ($mimeobj->parameters)) {
			$params = $mimeobj->parameters;
			for ($x = 0; $x < count($params); $x ++) {
				if(is_object($params) && isset($params->$x)) {
					$param = $params->$x;
				}elseif(is_array($params) && isset($params[$x])) {
					$param = $params[$x];
				}else {
					$param = null;
				}

				if(isset($param)) {
					if ((substr(strtolower($param->attribute),0,4) == 'name') && $param->value != '') {
						$name .= $param->value;//$this->enc_utf8($param->value);
					}
				}
			}
		}

		if (empty($name) && isset ($mimeobj->dparameters)) {
			$params = $mimeobj->dparameters;
			for ($x = 0; $x < count($params); $x ++) {
				if(is_object($params) && isset($params->$x)) {
					$param = $params->$x;
				}elseif(is_array($params) && isset($params[$x])) {
					$param = $params[$x];
				}else {
					$param = null;
				}

				if(isset($param)) {
					if (substr(strtolower($param->attribute),0,8) == 'filename' && $param->value != '') {
					//$name = $this->enc_utf8(quoted_printable_decode($param->value));
						$name .= $param->value;
					}
				}
			}
		}

		if(empty($name) && !empty($mimeobj->description)) {
			$name = $mimeobj->description;
		}

		/*if(substr($name,0,2)=='=?') {
			$name = $this->enc_utf8($name);
		}else {
			$start = strpos($name,"''");
			if($start) {
				$name = substr($name, $start);
			}
			$name = rawurldecode($name);
		}*/

		$x = 0;
		if (isset ($mimeobj->parts)) {
			for ($x = 0; $x < count($mimeobj->parts); $x ++) {
			// If we are in the root of the object increment by whole integers
				if ($section == 0) {
					/*if(strtolower($mimeobj->subtype)!='related' && strtolower($mimeobj->subtype)!='mixed' && strtolower($mimeobj->subtype)!='alternative')
					 {
						$nsection = $x +1;
						}else {
						//$nsection=0;
						}*/
					$nsection = $x +1;
				} elseif (
				($pos = strrpos($section, "."))
				//Uncommented this for mails from zimbra
				//&& ($mimeobj->parts[0]->type != TYPEMULTIPART || $mimeobj->parts[0]->subtype != 'RELATED')
				) {

					$subsection = (int) substr($section, $pos +1) + $x;
					//go_debug('sub '.$subsection);
					if ($subsection == '') {
						$subsection = '0';
					}
					$nsection = substr($section, 0, $pos).".". ($subsection +1);
				//go_debug('new '.$nsection);
				} else {
					$nsection = $section;
				}

				// If there are more parts to the part about to be processed reference it as a header with ".0"
				// but only if the child of this child isn't MULTIPART

				if (isset ($mimeobj->parts[$x]->parts) && count($mimeobj->parts[$x]->parts))// && strtolower($mimeobj->parts[$x]->subtype) != 'rfc822')
				{
				// Funny really, if a mime section is a inline message that has a multipart body you reference the message
				// mime section with "2" the inline message header with "2.0" and the subsections with	"2.x"
				// However if the mime section is a inline message with only 1 part then you reference the
				// mime section in the message with 2.0 and the	inline message body with 2.1

				//echo $mimeobj->parts[$x]->type.' '.TYPEMESSAGE.' ';
				//echo $mimeobj->parts[$x]->parts[0]->type.' '.TYPEMULTIPART.' '.$section.' '.$full_mime_type.'<br />';
				//if (!($mimeobj->parts[$x]->type == TYPEMESSAGE && $mimeobj->parts[$x]->parts[0]->type == TYPEMULTIPART)) {
					if ($mimeobj->parts[$x]->type != TYPEMESSAGE) {
					//echo 'ja';
						$nsection .= ".0";
					} else {
						$nsection .= "";
					}

				//$nsection .= ".0";
				}

				//added section is 0 check because novell uses message/rfc822 in the main headers
				if($section==0 || strtolower($full_mime_type) != 'message/rfc822') {
					$this->get_parts($mimeobj->parts[$x], $nsection, $mimeobj->subtype);
				}

				/*
				 * else {
					
				if (strtolower($mimeobj->parts[$x]->subtype) == 'mixed') {
				$section .= ".2";
				}
				}*/
			}
		}

		// If after processing the entire MIME object the $x variable is still zero then we didn't
		// process a multipart mime message.

		if ($x == 0 && $section == 0) {
			$section = "1";
		}

		if ($type != "multipart" && $full_mime_type) {
			/*if (eregi('message', $full_mime_type)) {
				$section ++;
				}*/

			$part_charset = 'ISO-8859-15';
			if ($mimeobj->ifparameters) {
				for ($x = 0; $x < count($mimeobj->parameters); $x ++) {
					$param = $mimeobj->parameters[$x];
					if ((strtolower($param->attribute) == 'charset') && ($param->value != '')) {
						$part_charset = $param->value;
						break;
					}
				}
			}

			//don't make an attachment for section=0 and type message/rfc822
			if(empty($name) && !empty($section) && strtolower($full_mime_type) == 'message/rfc822') {
				$name='Message.eml';
			}

			//echo $section.': '.$full_mime_type.' '.$name.'<br>';
			$bytes = isset ($mimeobj->bytes) ? $mimeobj->bytes : 0;
			$tmp = Array (
					'number' => $section,
					'id' => $mimeobj->ifid ? $mimeobj->id : 0,
					'name' => $this->enc_utf8($name),
					'mime' => $full_mime_type,
					'transfer' => $encoding,
					'charset' => $part_charset,
					'disposition' => $mimeobj->ifdisposition ? $mimeobj->disposition : '',
					'size' => $bytes,
					'human_size'=>Number::format_size($bytes),
					'type' => $parttype);

			array_unshift($this->message["parts"], $tmp);
		}
	}
	/**
	 * Check if a part is an attachment
	 *
	 * @param array $part returned by get_message or get_parts
	 * @return bool 
	 */
	function part_is_attachment($part)
	{
		return ((
						stripos($part["disposition"],"ATTACHMENT")!==false || ($part["name"] != '' && empty($part["id"]))
						&&
						!($part['type']=='APPLEDOUBLE' && $part['mime']== 'application/APPLEFILE') && strtolower($part['type'])!='alternative')
					||
				strtolower($part['mime'])=='text/calendar'
		);
	}
	
	/*function is_attachment($part)
	{
		return	(
		eregi("attachment", $part["disposition"]) ||
		($part['name']!='' && !eregi("inline", $part["disposition"])) ||
		($part["id"] == '' && eregi("inline", $part["disposition"]))
			
		)
		&& !eregi("message/RFC822", $part["mime"]);
	}*/
	
	/*
		function print_structure($mimeobj, $depth = 0, $section = 0)
		{
		for($y = 0; $y < $depth; $y++)
		{
		echo("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
		}
		echo($this->get_mime_type($mimeobj->type) . "/{$mimeobj->subtype},");
		echo($this->get_encoding($mimeobj->encoding) . "(<B>$section</B>)<br>");

		$x=0;
		if (isset($mimeobj->parts))
		{

		for($x = 0; $x < count($mimeobj->parts); $x++)
		{
		// If we are in the root of the object increment by whole	integers

		if($section == 0)
		{
		$nsection = $x + 1;
		$subsection = 0;
		// If we are in the object and the first sub-object of our	object isn't multipart
		// then increment the postfix by ".1" otherwise	we are multipart or a message
		// and leave the section id alone to be handled by the next	code block

		//else if(($pos = strrpos($section, ".")) && sizeof($mimeobj->parts) > 1)
		}else if(($pos = strrpos($section, ".")) && ($mimeobj->parts[0]->type != TYPEMULTIPART || $mimeobj->parts[0]->subtype != 'RELATED'))
		//}elseif($pos = strrpos($section, "."))
		{
		$subsection = (int) substr($section, $pos+1)+$x;
		if ($subsection == '')
		{
		$subsection = '0';
		}
		$nsection = substr($section, 0, $pos) .	"." . ($subsection + 1);
		}else
		{
		$nsection = $section;

		}

		// If there are more parts to the part about to be processed reference it as a header with ".0"
		// but only if the child of this child isn't MULTIPART

		if(isset($mimeobj->parts[$x]->parts) && count($mimeobj->parts[$x]->parts))
		{
		// Funny really, if a mime section is a inline message that has a multipart body you reference the message
		// mime section with "2" the inline	message header with "2.0" and the subsections with	"2.x"
		// However if the mime section is a inline message with only 1 part then you reference the
		// mime section in the message with 2.0 and the	inline message body with 2.1

		if(!($mimeobj->parts[$x]->type == TYPEMESSAGE && $mimeobj->parts[$x]->parts[0]->type ==	TYPEMULTIPART))
		{
		$nsection .= ".0";
		}else
		{
		$nsection .= "";
		}
		}

		$this->print_structure($mimeobj->parts[$x], $depth + 1, $nsection);
		}
		}

		// If after processing the entire MIME object the $x variable is still zero then we didn't
		// process a multipart mime message, it's just normal email so say so here.

		if($x == 0 && $section == 0)
		{
		echo($this->get_mime_type($mimeobj->type) . "/{$mimeobj->subtype}, ");
		echo($this->get_encoding($mimeobj->encoding) . "(<B>1</B>) (<B>NOT MIME MULTIPART</B>)<br>");
		}
		}
		*/

	/**
		* Get the encoding of a message part in text
		*
		* @param int $encoding The encoding type as ut comes from imap_fetch_structure()
		* @access private
		* @return String The Encoding
		*/

	function get_encoding($encoding) {
		switch ($encoding) {
			case 0 :
				$encoding = '7BIT';
				break;
			case 1 :
				$encoding = '8BIT';
				break;
			case 2 :
				$encoding = 'BINARY';
				break;
			case 3 :
				$encoding = 'BASE64';
				break;
			case 4 :
				$encoding = 'QUOTED-PRINTABLE';
				break;
			case 5 :
				$encoding = 'OTHER';
				break;
			default :
				$encoding = 'none';
				break;
		}

		return $encoding;
	}

	/**
		* Get the mime type of a message part in text
		*
		* @param int $type The mime type as ut comes from imap_fetch_structure()
		* @access private
		* @return String The mime type
		*/

	function get_mime_type($type) {
		switch ($type) {
			case 0 :
				$mime_type = 'text';
				break;
			case 1 :
				$mime_type = 'multipart';
				break;
			case 2 :
				$mime_type = 'message';
				break;
			case 3 :
				$mime_type = 'application';
				break;
			case 4 :
				$mime_type = 'audio';
				break;
			case 5 :
				$mime_type = 'image';
				break;
			case 6 :
				$mime_type = 'video';
				break;
			case 7 :
				$mime_type = 'model';
				break;
			default :
				$mime_type = 'unknown';
		}
		return $mime_type;
	}

	/**
		* Search a plain text header for a value
		*
		* @param String $fieldname The name of the field
		* @param String $header The plain text header to search through
		* @access private
		* @return string The value of the fieldname if found.
		*/

	function get_header_value($fieldname, $header) {
		$value=false;
		$results = array();
		if (preg_match("/\v$fieldname (.*)\v/", $header, $results)) {
			$value = trim($results[1]);
		}

		return $value;
	}

	/**
		* Delete messages from the IMAP server
		*
		* @param Array $messages An array of message UID's
		* @access public
		* @return void
		*/

	function delete($messages) {
		for ($i = 0; $i < count($messages); $i ++) {
			@ imap_delete($this->conn, $messages[$i], FT_UID);
		}
		return @ imap_expunge($this->conn);
	}





	/**
		* Return a message part
		*
		* @param int $uid The message UID
		* @param float $part_no The message part identifier
		* @param String $transfer The transfer-encoding
		* @param String $part_charset The character set of the part if applicable
		* @access public
		* @return string Message part data
		*/

	function view_part($uid, $part_no, $transfer, $part_charset = '', $peek=false) {
		global $GO_CONFIG;

		$charset='utf-8';
		
		$transfer=strtoupper($transfer);

		if($peek){
			$str = imap_fetchbody($this->conn, $uid, $part_no, FT_UID | FT_PEEK);
		}else
		{
			$str = imap_fetchbody($this->conn, $uid, $part_no, FT_UID);
		}
		

		if ($transfer == 'BASE64')
		{
			$str = imap_base64($str);
		}elseif ($transfer == 'QUOTED-PRINTABLE')
		{
			$str = quoted_printable_decode($str);
		}

		$part_charset=strtolower($part_charset);


		//PHP sets us-ascii when the charset is unknown. Try to detect it.
		if($part_charset=='us-ascii')
		{
			if(function_exists('mb_detect_encoding'))
			{
				$part_charset = mb_detect_encoding($str, "auto");
				if(!$part_charset){
					$part_charset='us-ascii';
				}
			}else
			{
				$part_charset = 'ISO-8859-1';
			}
		}
//go_debug($part_charset);
		if ($part_charset != '') {			
			$str = String::clean_utf8($str, $part_charset);
			if($part_charset != $charset){
				$str = str_replace($part_charset, $charset, $str);
			}
		}
		
		return $str;
	}

	function f($name) {
		$value = isset ($this->message[$name]) ? $this->message[$name] : false;
		return $value;
	}

	/**
		* Get the delimiter that is used to delimit Mailbox names
		*
		* @access public
		* @return mixed The delimiter or false on failure
		*/

	function get_mailbox_delimiter() {
		$list = imap_getmailboxes($this->conn, "{".$this->connectstring."}", '%');
		if (is_array($list)) {
			$folder = array_shift($list);
			if (strlen($folder->delimiter) > 0) {
				return $folder->delimiter;
			}
		}
		return false;
	}

	/**
		* Check if the given mailbox root is valid and return it with the correct delimiter
		*
		* @param $mbroot The Mailbox root. (eg. INBOX/)
		* @access public
		* @return mixed Mailbox root with delimiter or false on failure
		*/

	function check_mbroot($mbroot) {
		$mbroot = trim($mbroot);

		$list = imap_getmailboxes($this->conn, "{".$this->connectstring."}", '%');
		if (is_array($list)) {
			while ($folder = array_shift($list)) {
				if (!isset ($delimiter) && strlen($folder->delimiter) > 0) {
					$delimiter = $folder->delimiter;
					if (substr($mbroot, -1) == $delimiter) {
						$mbroot = substr($mbroot, 0, -1);
					}
				}

				if (str_replace("{".$this->connectstring."}", "", $folder->name) == $mbroot) {
					return $mbroot.$delimiter;
				}
			}
		}
		return false;
	}

	/**
		* Return all mailboxes in an array with name, delimiter and attributes
		*
		* @param String $mailbox_root The mailbox root
		* @param bool $name_only Return only the mailbox names in the result
		* @access public
		* @return array The mailboxes
		*/
	function get_mailboxes($mailbox_root = '', $name_only = false) {

		$this->mailboxes = array ();
		$list = imap_getmailboxes($this->conn, "{".$this->connectstring."}", $mailbox_root.'*');
		if (is_array($list)) {
			foreach ($list as $value) {
				if (substr($value->name, -1) != $value->delimiter && strlen($value->delimiter) > 0) {
					

					$mailbox['name'] = str_replace("{".$this->connectstring."}", "", $value->name);

					/*
					 * This will make sure only root.abc is returned and not rootabc
					 */

					if(!empty($mailbox_root) && substr($mailbox_root,-1,1)!=$value->delimiter){
						$mailbox_root .= $value->delimiter;
					}

					if(empty($mailbox_root) || strpos($mailbox['name'], $mailbox_root)===0){

						$mailbox['delimiter'] = $value->delimiter;
						$mailbox['attributes'] = $value->attributes;

						if ($name_only) {
							$this->mailboxes[] = $mailbox['name'];
						} else {
							$this->mailboxes[] = $mailbox;
						}
					}

					//sometimes the IMAP server returns the parent folder as well so avoid a loop
					/*if($mailbox['name']!=substr($mailbox_root,0, -strlen($mailbox['delimiter'])))
					{
						$this->mailboxes[] = $mailbox;
						if (!($mailbox['attributes'] & LATT_NOINFERIORS)) {
							$this->mailboxes = array_merge($this->mailboxes, $this->get_mailboxes($mailbox['name'].$mailbox['delimiter'],$name_only));
						}
					}*/

				}
			}
		}
		sort($this->mailboxes);
		
		return $this->mailboxes;
	}

	/**
		* Return all subscribed mailboxes in an array with name, delimiter and attributes
		*
		* @param String $mailbox_root The mailbox root
		* @param bool $name_only Return only the mailbox names in the result
		* @access public
		* @return array The subscribed mailboxes
		*/
	function get_subscribed($mailbox_root = '', $name_only = false) {
		$this->mailboxes = array ();
		$list = imap_getsubscribed($this->conn, "{".$this->connectstring."}", $mailbox_root.'*');
//go_debug($list);
		if (is_array($list)) {
			foreach ($list as $value) {
				if (substr($value->name, -1) != $value->delimiter && strlen($value->delimiter) > 0) {														
					$mailbox['name'] = str_replace("{".$this->connectstring."}", "",$value->name);
					
					
					/*
					 * This will make sure only root.abc is returned and not rootabc
					 */

					if(!empty($mailbox_root) && substr($mailbox_root,-1,1)!=$value->delimiter){
						$mailbox_root .= $value->delimiter;
					}


					//sometimes the IMAP server returns the parent folder as well so avoid a loop					
					if(empty($mailbox_root) || strpos($mailbox['name'], $mailbox_root)===0){
						$mailbox['delimiter'] = $value->delimiter;
						$mailbox['attributes'] = $value->attributes;

						if ($name_only) {
							$this->mailboxes[] = $mailbox['name'];
						} else {
							$this->mailboxes[] = $mailbox;
						}
						
						/*if (!($mailbox['attributes'] & LATT_NOINFERIORS)) {
							$this->mailboxes = array_merge($this->mailboxes, $this->get_subscribed($mailbox['name'].$mailbox['delimiter'], $name_only));
						}*/
					}
				}
			}
		}
		sort($this->mailboxes);
		return $this->mailboxes;
	}

	/**
		* Check if a mailbox is subscribed
		*
		* @param String $name The name of the mailbox
		* @param String $mailbox_root The mailbox root
		* @access public
		* @return bool
		*/
	function is_subscribed($name, $mailbox_root) {
		$this->get_subscribed($mailbox_root);
		for ($i = 0; $i < count($this->mailboxes); $i ++) {
			if ($this->mailboxes[$i]['name'] == $name) {
				return true;
			}
		}
		return false;
	}

	/**
		* Subscribe a mailbox
		*
		* @param String $name The name of the mailbox
		* @access public
		* @return bool True on success
		*/
	function subscribe($name) {
		return imap_subscribe($this->conn, "{".$this->connectstring."}".$name);
	}

	/**
		* Unsubscribe a mailbox
		*
		* @param String $name The name of the mailbox
		* @access public
		* @return bool True on success
		*/
	function unsubscribe($name) {
		return imap_unsubscribe($this->conn, "{".$this->connectstring."}".$name);
	}

	/**
		* Delete a mailbox
		*
		* @param String $name The name of the mailbox
		* @param String $mailbox_root The mailbox root
		* @access public
		* @return bool True on success
		*/
	function delete_folder($name, $mailbox_root) {		
		$children = $this->get_mailboxes($name);
		rsort($children);

		$children[]=array('name'=>$name);
		
		foreach($children as $child)		
		{
			if(!imap_deletemailbox($this->conn, "{".$this->connectstring."}".$child['name']))
			{
				return false;
			}
		}
		return true;
	}

	/**
		* Create a mailbox
		*
		* @param String $name The name of the mailbox
		* @access public
		* @return bool True on success
		*/
	function create_folder($name, $delimiter='.') {

		//echo imap_utf7_encode(utf8_decode($name)).' -> '.$this->utf7_encode($name);

		if (imap_createmailbox($this->conn, "{".$this->connectstring."}".$name)) {
			return $this->subscribe($name);
		}
	}

	/**
		* Rename a mailbox
		*
		* @param String $old_name The current name of the mailbox
		* @param String $new_name The new name of the mailbox
		* @access public
		* @return bool True on success
		*/
	 function rename_folder($old_name, $new_name) {

		 $children = $this->get_mailboxes($old_name);

		 //debug('Renaming: '.$old_name.' to: '.$new_name);

		 //debug($children);

		 $ret = imap_renamemailbox($this->conn, "{".$this->connectstring."}".$old_name, "{".$this->connectstring."}".$new_name);
		 if(!$ret) {
			 return false;
		 }
		 $this->unsubscribe($old_name);
		 $this->subscribe($new_name);

		 foreach($children as $old_child) {
			 $old_child = $old_child['name'];
			 $pos = strpos($old_child, $old_name);
			 $new_child = substr_replace($old_child, $new_name, $pos, strlen($old_name));
			 //debug('Renaming: '.$old_child.' to: '.$new_child);
			 $this->unsubscribe($old_child);			 
			 //if(imap_renamemailbox($this->conn, "{".$this->connectstring."}".$old_child, "{".$this->connectstring."}".$new_child)) {
				 //debug('Subscribing:'.$new_child);
				 $this->subscribe($new_child);
			 //}
		 }

		 return true;
	 }

	/**
		* Move messages to another mailbox
		*
		* @param String $folder The mailbox where the messages need to go
		* @param Array $messages An array of message UID's to move
		* @access public
		* @return bool True on success
		*/
	function move($folder, $messages, $expunge=true) {
		$messageset = implode(",", $messages);
		if (imap_mail_move($this->conn, $messageset, $folder, CP_UID)) {
			
			if(!in_array($folder, $this->touched_folders))
			{
				$this->touched_folders[]=$folder;
			}
			
			if($expunge)
				$this->expunge();
			
			return true;
		}
		return false;
	}
	
	function expunge()
	{
		imap_expunge($this->conn);
		//go_debug('Expunged');
	}

	/**
		* Append a message to a maibox
		*
		* @param String $mailbox The mailbox where the message needs to go
		* @param String $body The message body
		* @param Int $flags The message flags (Unseen, Replied, flagged)
		* @access public
		* @return bool True on success
		*/
	function append_message($mailbox, $body, $flags = "") {
		return @ imap_append($this->conn, "{".$this->connectstring."}".$mailbox, $body, $flags);
	}

	/**
		* Set message flags
		*
		* @param String $mailbox The mailbox where the message is
		* @param String $uid_array An array of message UID's
		* @param Int $flags The message flags (Unseen, Replied, flagged)
		* @param String $action If action is reset the given flags will be cleared
		* @access public
		* @return bool True on success
		*/
	function set_message_flag($mailbox = "INBOX", $uid_array, $flags, $action = "") {
		if ($mailbox == $this->mailbox || $this->reopen($mailbox)) {
			$msgno_set = implode(",", $uid_array);

			if ($action == "reset") {
				if (imap_clearflag_full($this->conn, $msgno_set, $flags, ST_UID)) {
					return true;
				} else {
					return false;
				}
			} else {
				if (imap_setflag_full($this->conn, $msgno_set, $flags, ST_UID)) {
					return true;
				} else {
					return false;
				}
			}
		} else {
			return false;
		}
	}

	/**
		* Re-open a mailbox
		*
		* @param String $mailbox The mailbox to open
		* @param Int $flags Connection flags see imap_open() PHP docs.
		* @access public
		* @return bool True on success
		*/
	function reopen($mailbox = "INBOX", $flags = "") {

		if ($mailbox==$this->mailbox || imap_reopen($this->conn, "{".$this->connectstring."}".$mailbox, $flags)) {
			$this->mailbox = $mailbox;
			return true;
		} else {
			return false;
		}
	}

	/**
		* Get mailbox message info and put it in $this->mailbox_info.
		*
		* @access public
		* @return bool True on success
		*/
	function mailbox_info() {
		$info = imap_mailboxmsginfo($this->conn);
		if ($info) {
			$this->mailbox_info = array ();
			$this->mailbox_info["date"] = $info->Date;
			$this->mailbox_info["driver"] = $info->Driver;
			$this->mailbox_info["mailbox"] = $info->Mailbox;
			$this->mailbox_info["nmsgs"] = $info->Nmsgs;
			$this->mailbox_info["recent"] = $info->Recent;
			$this->mailbox_info["size"] = $info->Size;
			return true;
		} else {
			return false;
		}
	}


	function enc_utf8($str) {
		//some mail clients create encoded strings such: =?iso-8859-1?Q? "Andr=E9=20Mc=20Intyre" ?=
		//containing space values inside, but they mustn't. The space values have to be removed before
		//they are going to be converted to utf8.

		//go_debug($str);

		if (strpos($str, '=?') !== false) {

			$obj = imap_mime_header_decode($str);
			//go_debug($obj);
			$decoded='';
			for ($i=0; $i<count($obj); $i++) {
				if($obj[$i]->charset!='UTF-8' && $obj[$i]->charset!='default'){
					$decoded.=iconv($obj[$i]->charset, 'UTF-8', $obj[$i]->text);
				}else
				{
					$decoded.=$obj[$i]->text;
				}
			}
			//go_debug($decoded);
			return $decoded;

			/*$decoded = preg_replace_callback(
        '/=\?.*\?=/U',
        create_function(
            // single quotes are essential here,
            // or alternative escape all $ as \$
            '$matches',
            'go_debug($matches[0]);
	$decoded= iconv_mime_decode($matches[0],ICONV_MIME_DECODE_CONTINUE_ON_ERROR,\'UTF-8\');

			if($decoded){
				return $decoded;
			}else
			{
				return imap_utf8($matches[0]);
			}
	'
        ),
        $str
			);

			//$str = str_replace(" ", "", $str);
			//return imap_utf8($str);
			//$decoded = iconv_mime_decode($str,ICONV_MIME_DECODE_CONTINUE_ON_ERROR,'UTF-8');
			//go_debug($decoded);
			if($decoded){
				return $decoded;
			}else
			{
				return imap_utf8($str);
			}*/
		}else {
			if (function_exists('iconv'))
			{
				if($converted = @iconv('ISO-8859-15', 'UTF-8//IGNORE', $str))
				{
					return $converted;
				}
			}
		}
		return $str;

		/*
		 $text = '';
		 if($elements = imap_mime_header_decode($str))
		 {
		 foreach($elements as $element)
		 {
			$text .= $element->text;
			}
			return utf8_encode($text);
			}	*/
	}


	function encode_imap_path($str, $delimiter='.')
	{
		$encoded = array();
		$mailboxes = explode($delimiter, $str);
		foreach($mailboxes as $mailbox)
		{
			$encoded[]=$this->utf7_encode($mailbox);
		}
		return implode($delimiter, $encoded);
	}

	function decode_imap_path($str, $delimiter='.')
	{
		$decoded = array();
		$mailboxes = explode($delimiter, $str);
		foreach($mailboxes as $mailbox)
		{
			$decoded[]=$this->utf7_decode($mailbox);
		}
		return implode($delimiter, $decoded);
	}

	function utf7_imap_encode($input)
	{
		$charset = 'UTF-8';
		
		if(function_exists('mb_convert_encoding'))
		{
			return mb_convert_encoding($input,"UTF7-IMAP",$charset);
		}elseif (function_exists('iconv')) {
			if($converted_input = @iconv($charset, 'UTF-7', $input))
			{
				$input=preg_replace("/&/","&-",$converted_input);
				$input=preg_replace("/&/","&-",$converted_input);
				$input=preg_replace("/\+A/","&A",$input);
				$input=preg_replace("/&ACY /","&- ",$input);
				$input=preg_replace("/&ACY/","&",$input);
				$input=preg_replace("/&AF8(-)?/","_",$input);
				$input=preg_replace("/&ADs(-)?/",";",$input);				
				$input=preg_replace("/&AFs(-)?/","[",$input);
				$input=preg_replace("/&AF0(-)?/","]",$input);
				return $input;
			}
		}
				
		//doesnt work well with UTF-8
		return @imap_utf7_encode(utf8_decode($input));
	}



	function utf7_imap_decode($input)
	{
		$charset='UTF-8';

		if(function_exists('mb_convert_encoding'))
		{
			return mb_convert_encoding($input,$charset, "UTF7-IMAP");
		}elseif (function_exists('iconv')) {
			$input=preg_replace("/&A/","+A",$input);
			$input=preg_replace("/&-/","&",$input);

			if($converted_input = @iconv('UTF-7', $charset, $input))
			{
				return $converted_input;
			}
		}
		return utf8_encode(@imap_utf7_decode($input));
	}
	
	
	function extract_uuencoded_attachments(&$body)
	{
		$body = str_replace("\r", '', $body);
		$regex = "/(begin ([0-7]{3}) (.+))\n(.+)\nend/Us";
    
		preg_match_all($regex, $body, $matches);
      
    $attachments = array();
        
    for ($i = 0; $i < count($matches[3]); $i++) {
			$boundary	= $matches[1][$i];
			$fileperm	= $matches[2][$i];
			$filename	= $matches[3][$i];
			
			$size = strlen($matches[4][$i]);
			
			$attachments[]=array(
				'boundary'=>$matches[1][$i],				
				'permissions'=>$matches[2][$i],
				'name'=>$matches[3][$i],
				'data'=>$matches[4][$i],
				'disposition'=>'ATTACHMENT',
				'type'=>'',
				'size'=>$size,
				'human_size'=>Number::format_size($size),
				'mime'=>File::get_mime($matches[3][$i])			
			);
    }
    
    //remove it from the body.
    $body = preg_replace($regex, "", $body);
    //go_debug($body);
    
    return $attachments;
	}



}
